# Copyright (c) 2005, Reinhard Tartler, Stefan Potyra 
#
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
# 
#     * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#


import os, sys
from mod_python import psp
from mod_python import util
from mod_python.Session import Session

from common import *
from Uploads import nuke_upload
import User
import Package
from Config import Config

#TMPL_DIR = "/home/siretart/public_html/revu/templates"
LISTING_TMPL = "listing.html"
DETAILS_TMPL = "details.html"

def _tmpl_path(name):
	return os.path.join(cfg.getTemplatesPath(), name)

def _rewrite_template(name, vars=None):
	""" sparse version of PSP which can substitute variables from 
            template files.
	    name: name of the template
	    vars: variables to replace (as dict)
	    return: string of replaced template
	"""
	f = file(_tmpl_path(name), "r")
	l = f.read()
	f.close()

	if vars is None:
		return l

	for key, value in vars.iteritems():
		l = l.replace("<%%=%s%%>" % key, value)
	return l

def _actionlinks(level, upid, isArchived):
	if level == 'Admin' or level == 'Reviewer':
		if isArchived == 'true':
			archive = "unarchive" 
		else:
			archive = "archive"
		html = []
		html.append('<a href="index.py?%s=%d">[%s]</a>' % (archive, upid, archive.title()))
		if (level == 'Admin') and (isArchived == 'true'): # lets see how to implement this
			html.append('<a href="index.py?nuke=%d">[Nuke]</a>' % upid)
		return " ".join(html)
	else: 
		return ''

def _imgs(archived, needswork, new, needsreview, ready, advocates):
	""" generate some images about the package 
	"""

	if archived=='t':
		return '<img src="images/archive.png" alt="archived" title="Archived">'

        elif needswork:
                return '<img src="images/needswork.png" alt="needs work" title="needs work">'

	elif advocates > 1:
		return '<img src="images/heart.png" alt="ready for upload" title="Ready for upload">'

	elif advocates == 1:
		return '<img src="images/bulb.png" alt="advocated" title="Advocated">'

	elif needsreview:
		return '<img src="images/xmag.png" alt="needs review" title="Needs review">'


def _upload_entry(row, col, level, showArchived, anchor):
	""" return one entry from the result tuple as html
            table entry. col should alternate between 0 and 1.
	"""
	( name, email, date, isnew)   = ( row[0], row[1], row[2], row[3] )
	( upid, isinArchive ) = ( row[4], row[5] )
	comments = row[6]
	advocates = row[7]
	negativeAdvocates = row[8]

	needswork = negativeAdvocates > 0
	ready = advocates > 1
	needsReview = (negativeAdvocates == 0) and (not ready)
	
	s = """
	<tr class="row%d">
	<td style="text-align: center;">
	    %s
	    %s
	</td>
	<td>
	    <a class="good" href="details.py?package=%s"> %s </a>
	</td>
	<td> %s </td>
	<td> %s </td>
	<td> %s (%d) </td>
	<td> %s </td> 
	</tr>
	""" % ( col + 1, anchor,
		_imgs(isinArchive, needswork, isnew, needsReview, ready, advocates),
		name, name, display_email(email), date, comments, 
		advocates, _actionlinks(level, upid, showArchived))
	return s

def _makeHeader(isNew):
	if isNew:
		headervars = { "title": "New packages" }
	else:
		headervars = { "title": "Updated packages"}
	header = _rewrite_template("listheader.html", vars=headervars)
	
	return header


def _uploads_rows(req, c, level, showArchived='false'):
	sql = """
		SELECT sp.name, us.email, to_char(up.dateofupload at time zone 'Europe/Berlin', 'Month DD  HH24:MI') as dateou,
			sp.isNew, 
			up.upid, isinarchive,
			( SELECT count(*) FROM Comments where upid=up.upid ) AS comments, 
			( SELECT count(*) FROM Comments where upid=up.upid AND isadvocating=true ) AS advocates,
			( SELECT count(usid) FROM Comments NATURAL JOIN users
				WHERE upid=up.upid 
					AND isadvocating=false 
					AND Comments.usid NOT IN (
						SELECT usid FROM Comments 
						WHERE upid=up.upid AND isadvocating=True
					)
					AND users.accesslevel IN ('Admin', 'Reviewer')
					AND usid != up.usid
			) AS negativeAdvocates
		FROM
			( SELECT max(up.dateofupload) as maxdate,
				 sp.sid
			  FROM  SourcePackage sp INNER JOIN Upload up ON sp.sid=up.sid
			  GROUP BY sp.sid, sp.name, sp.isnew, sp.isinarchive ) AS ug
		INNER JOIN Upload up ON (ug.maxdate=up.dateofupload AND ug.sid=up.sid)
		INNER JOIN SourcePackage sp ON up.sid=sp.sid
		INNER JOIN Users us ON up.usid=us.usid
		WHERE up.isarchived = %s
		ORDER BY isNew DESC, negativeAdvocates ASC, advocates DESC, dateofupload ASC;
	""" % showArchived

	r = c.query(sql)
	rows = []
	col = 0

	footervars = {}
	footer = _rewrite_template("listfooter.html", vars=footervars)

	state = 0
	needswork = False
	anchor = ""
	for row in r.getresult():
		isNew = row[3] == 't'

		if state == 0:
			rows.append(_makeHeader(isNew))
			if isNew:
				state = 1
			else:
				state = 2
		elif state == 1:
			if not isNew:
				# new table for updated packages packages
				rows.append(footer)
				rows.append(_makeHeader(isNew))
				state = 3

		# anchor for needswork packages
		if row[8] > 0 and not needswork:
			anchor='<a name="needsworksection"></a>\n'
			needswork = True
			
		col = 1 - col
		s = _upload_entry(row, col, level, showArchived, anchor)
		anchor = ""
		rows.append(s)

	body = "\n".join(rows)

	return body + footer

def _login_form(req, message=''):
	tmpl = _tmpl_path('loginform.html')
	m = psp.PSP(req, tmpl, vars={'message': message} )
	return m

def _welcome_user(req, u):
	tmpl = _tmpl_path('welcome.html')
	return psp.PSP(req, tmpl, vars={'email': u.email})

def _footer(req, session):
	return ''
	return '<small>%s</small>' % session.id()

def _archive(c, upid):
	sql = "UPDATE Upload SET isArchived = true WHERE upid = %d " % upid
	c.query(sql)

def _unarchive(c, upid):
	sql = "UPDATE Upload SET isArchived = false WHERE upid = %d " % upid
	c.query(sql)

def index(req):
	"""Called by default from apache
	"""
	s = Session(req, timeout=43200)
	form = req.form
	save = False
	global cfg
	cfg = Config(req)

	if s.is_new():
		login = _login_form(req)
	else:
		s.load()
		
	if form.has_key('email') and form.has_key('password'):
		s['email'] = form['email']
		s['password'] = form['password']
		s.save()

	if form.has_key('logout'):
		s.invalidate()

	level = None
	c = cfg.getConnection()
	if s.has_key('email') and s.has_key('password'):
		u = User.User(s['email'], s['password'], c)
		if u.isAuthed():
			login = _welcome_user(req, u)
			level = u.accesslevel
		else:
			email = s['email']
			recoverlink = '<a href="lostpw.py?email=%s">recover</a>' % email
			login = _login_form(req, 'Login for user "%s" failed, please retry or %s.' % (email, recoverlink))
	else:
		login = _login_form(req)

	fs = util.FieldStorage(req)
	if fs.has_key('archive') and ( level == 'Admin' or level == 'Reviewer' ):
		_archive(c, int(fs['archive']))

	if fs.has_key('unarchive') and ( level == 'Admin' or level == 'Reviewer' ):
		_unarchive(c, int(fs['unarchive']))

	if fs.has_key('nuke') and (level == 'Admin'):
		nuke_upload(cfg, c, int(fs['nuke']))

	if fs.has_key('archived') and fs['archived'] in ['true', 'false']:
		showArchived = fs['archived']
	else:
	    	showArchived = 'false'

	if showArchived == 'false':
		title = 'Current'
		link = '<a href="index.py?archived=true">archived</a>'
	else:
		title = 'Archived'
		link = '<a href="index.py?archived=false">current</a>'

	if fs.has_key('archived') and fs['archived'] in ['true', 'false']:
		showArchived = fs['archived']
	else:
	    	showArchived = 'false'

	if showArchived == 'false':
		title = 'Current'
		link = '<a href="index.py?archived=true">Show archived uploads</a>'
	else:
		title = 'Archived'
		link = '<a href="index.py?archived=false">Show current uploads</a>'

	vars = { "login": login,
		 "title": title,
		 "link": link,
		 "uploads": _uploads_rows(req, c, level, showArchived=showArchived),
		 "footer" : _footer(req, s)}
		 
	s.save()

	tmpl = _tmpl_path('listings.html')
	m = psp.PSP(req, tmpl, vars=vars)
	return m
	
