#!/usr/bin/python
#
# Copyright (c) 2005 Reinhard Tartler, Stefan Potyra
# Copyright (c) 2008 Siegfried-Angel Gevatter Pujals <rainct@ubuntu.com>
#
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
# 
#     * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys
import os
from ConfigParser import SafeConfigParser, NoSectionError, NoOptionError
from optparse import OptionParser

from Connection import Connection

class Config:
    
    def __init__(self, req=None):
        """ params: req optional: apache request
        """
        if req:
            self.basepath = req.get_options()['REVU_BASE']
        else:
            self.basepath = self._determineBasePath()
        self._connection = None
        self._config_file = os.path.join(self.basepath, 'config/revu.cfg')
        self._readConfigFile()
    
    def _readConfigFile(self):
        """ protected
            evaluate the config file 
            param releaseConfig: release the config reader after reading
        """
        
        self._scp = SafeConfigParser()
        self._scp.read(self._config_file)
        
        self._dbName = self._scp.get("database", "Database")
        self._dbUser = self._scp.get("database", "User")
        self._dbPasswd = self._scp.get("database", "Password")
        self._lpUploaders = self._scp.get("launchpad", "Uploaders")
        self._incomingdir = self._scp.get("global", "incomingdir")
        self._baseurlpath = self._scp.get("global", "baseurlpathquoted")[1:-1]
        self._removelogfile = self._scp.get("global", "removelogfile")
        self._templates = self._scp.get("global", "templates")
        self._expire_age = self._scp.get("global", "expire_age")
    
    def getConnection(self):
        """ Config.getConnection() -> obj
        
        Return a Connection object. If necessary, create it first.
        
        """
        
        if self._connection is None:
            self._connection = Connection(self._dbName,
                self._dbUser, self._dbPasswd)
        return self._connection
    
    def _determineBasePath(self):
        """ Dirty hack to determine the basepath of REVU. """
        path = os.path.dirname(os.path.realpath(sys.argv[0]))
        if path.endswith('/scripts') or path.endswith('/bin'):
            basepath = path[:path.rfind('/')]
        else:
            basepath = path
        return basepath
    
    def getBasePath(self):
        return self.basepath
    
    def getTemplatesPath(self, file=''):
        return os.path.join(self.getBasePath(), self._templates, file)
    
    def getBaseUrlPath(self):
        return self._baseurlpath
    
    def getIncomingDir(self):
        return self._incomingdir
    
    def getRemoveLogFile(self):
        return self._removelogfile
    
    def getLaunchpadUploaders(self):
        return self._lpUploaders
    
    def getExpireAge(self):
    	return self._expire_age

# This file can either be imported into another Python file or called directly
# from a shell script in order to read the configuration.
if __name__ == "__main__":
    
    parser = OptionParser()
    parser.add_option("-g", "--get-option", dest="option", type="string",
                      help="get an option from the configuration file")
    parser.add_option("-s", "--section", dest="section", default="global",
                      help="select section where the option resides; " + \
                      "default is global", type="string")
    
    (options, args) = parser.parse_args()
    if options.option is None:
        parser.print_help()
        parser.error('You must specify which value you want to get.\n')
    
    cfg = Config()
    
    try:
        print cfg._scp.get(options.section, options.option)
        sys.exit(0)
    except NoOptionError:
        sys.stderr.write('%s: Could not find option "%s" in Section [%s].\n' % 
            (cfg._config_file, options.option, options.section))
        sys.exit(2)
    except NoSectionError:
        sys.stderr.write('%s: Could not find section [%s].\n' %
            (cfg._config_file, options.section))
        sys.exit(2)
