# Copyright (c) 2005 Reinhard Tartler, Stefan Potyra 
#
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
# 
#     * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

from Config import Config

def uploaddir(c, upid):
        sql = """SELECT name, to_char(dateofupload, 'YYMMDDHH24MI') as date
                 FROM Upload u JOIN SourcePackage sp ON u.sid = sp.sid
                 WHERE upid = %d
              """ % upid
        r = c.query(sql)
        if r.ntuples() == 1:
                return '-'.join([r.dictresult()[0]['name'], r.dictresult()[0]['date']])
        return ''

def upload_date(c, upid):
	sql = """SELECT to_char(dateofupload, 'Month DD  HH24:MI') FROM Upload WHERE upid = %d
	      """ % upid
	r = c.query(sql)
	if r.ntuples() == 1:
		return r.getresult()[0][0]

def basedir(c, upid):
	""" Returns the path on the local filesystem to the incoming dir of a source package 
	    relative to the INCOMING_DIR
	"""
	return uploaddir(c, upid) + '/'

def nuke_upload_rec(cfg, c, upid):
	""" removes all comments and upload information from all uploads which
	match the same source package as the upload referenced by upload id upid
	"""
	logfile = cfg.getRemoveLogFile()
	f = open(logfile, 'a')

	sql = "SELECT name FROM sourcepackage s INNER JOIN upload u ON u.sid=s.sid WHERE u.upid = %d" % upid
	r = c.query(sql)

	if r.ntuples() == 1:
		name = r.getresult()[0][0]
		s = 'rm -rf %s/%s-* # all uploads of source package %s' % (cfg.getIncomingDir(), name, name)
		
		print s
		f.write(s + '\n')
		f.close()
	else:
		s = "something went wrong while trying to recursively delete upload %d" % upid
		print s
		f.write(s + '\n')
		f.close()
		return

	sql = """DELETE FROM comments WHERE upid IN 
	                (SELECT upid FROM upload WHERE sid in 
                                (SELECT sid FROM upload WHERE upid = %d))""" % upid
	c.query(sql)
	sql = """DELETE FROM Upload WHERE upid IN 
	                (SELECT upid FROM upload WHERE sid in 
                                (SELECT sid FROM upload WHERE upid = %d))""" % upid
	return c.query(sql)

def nuke_upload(cfg, c, upid):
	logfile = cfg.getRemoveLogFile()
	""" removes all comments concerning an upload and the upload itself """
        sql = "DELETE FROM Comments WHERE upid = %d" % upid
        c.query(sql)
        sql = "DELETE FROM Upload WHERE upid = %d" % upid

	f = open(logfile, 'a')
	s = 'rm -rf %s/%s # was upload with upid: %d' % (cfg.getIncomingDir(), basedir(c,upid), upid)
	print s
	f.write(s + '\n')
	f.close()

        r = c.query(sql)
	return r
