#!/usr/bin/python
#
# Copyright (c) 2008 Siegfried-Angel Gevatter Pujals <rainct@ubuntu.com>
#
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
# 
#     * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

from Config import Config
from common import *

def main():
    """ main -> None
    
    This is cleanup.py's main function, intended to be called by a cron job.
    It's function is to automatically archive uploads that have been inactive
    for a long period of time.
    
    """
    
    # Connect to the database
    config = Config()
    connection = config.getConnection()
    
    # Define some convenience aliases
    query = connection.query_result
    age = config.getExpireAge()
    
    # Get the last upload for each source package
    uploads = query("""
        SELECT Upload.upid, Upload.sid 
        FROM (SELECT MAX(Upload.upid) as upid FROM
        (SELECT sid FROM SourcePackage WHERE NOT isinarchive) AS x
        INNER JOIN Upload ON Upload.sid = x.sid GROUP BY Upload.sid)
        AS x INNER JOIN Upload ON Upload.upid = x.upid
        WHERE NOT isarchived AND dateofupload < (NOW() - INTERVAL '%s')
        """ % age)
    
    for (upid, sid) in uploads:
        last_comment = query("SELECT dateofcomment FROM Comments " +\
            "WHERE upid='%d' AND dateofcomment > (NOW() - INTERVAL '%s')" \
            % (upid, age))
        
        if not last_comment:
            print 'Source Package: %d, Upload: %d' % (sid, upid)

if __name__ == '__main__':
    
    try:
        main()
    
    except KeyboardInterrupt:
        sys.exit(1)
