#!/usr/bin/env python

import sys
import RDF
import os
import shutil
from optparse import OptionParser

class RevuKeyUpdater:
    """ Retrieves Fingerprints from a Launchpad group by parsing the group's RDF
       and stores the corresponding keys in a new GPG keyring. """
    
    def __init__(self, lpgroup, revubase):
        "Initializes this class."
        self.lpgroup = lpgroup
        self.keyserver = 'keyserver.ubuntu.com'
        self.keyring = "%s/launchpad.gpg" % revubase
        self.gpgopts = "--no-options --no-default-keyring "
        self.gpgopts += "--secret-keyring %s/secring.gpg" % revubase
        self.gpgopts += " --trustdb-name %s/trustdb.gpg" % revubase
        self.gpgopts += " --keyring %s" % self.keyring
        self.gpgopts += " --keyserver keyserver.ubuntu.com"
        self.gpgopts += " --no-auto-check-trustdb --trust-model always"
        self.fingerprints = []
    
    def get_fingerprints(self):
        "Parses fingerprints from the RDF."
        model = RDF.Model()
        model.load('https://launchpad.net/people/%s/+rdf' % self.lpgroup)
        q = RDF.Query("SELECT ?fingerprint WHERE (?any, <http://xmlns.com/wot/0.1/fingerprint>, ?fingerprint )")
        
        results = q.execute(model)
        for statement in results:
            self.fingerprints.append(statement['fingerprint'].literal_value['string'])
    
    def do_update(self):
        "Handles calling GPG for all found fingerprints."
        self.get_fingerprints()
        for fingerprint in self.fingerprints:
            # get the keyid from the fingerprint
            key = fingerprint[-8:]
            os.system("gpg %s --recv-key %s" % (self.gpgopts, key))

def main():
    parser = OptionParser()
    parser.add_option("-g", "--launchpad-group", dest="group", type="string")
    parser.add_option("-b", "--revu-base", dest="base", type="string")
    (options, args) = parser.parse_args()
    
    updater = RevuKeyUpdater(lpgroup = options.group or 'revu-uploaders',
        revubase = options.base or '/srv/revu-production')
    updater.do_update()

if __name__ == '__main__':
	try:
		main()
	except KeyboardInterrupt:
		print 'User requested interrupt'
		sys.exit(1)
