#!/usr/bin/python
#
# Copyright (c) 2005, Reinhard Tartler, Stefan Potyra 
#
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
# 
#     * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

from optparse import OptionParser
from changes import ChangesParser


def print_files(changes):
    for row in changes[0]['files']:
        print row[4]

def print_version(changes):
    print changes[0]['version']

def print_uploader(changes):
    print changes[0]['changed-by']

def print_name(changes):
    print changes[0]['source']

if __name__ == '__main__':
    usage = "usage: %prog [options]"
    parser = OptionParser(usage)
    parser.add_option("-c", "--changes", dest="changes",
              help="changesfile to query")
    parser.add_option("-f", action="store_true", dest="files",
              help="show filenames of the source package")
    parser.add_option("-v", action="store_true", dest="version",
              help="show version of the source package")
    parser.add_option("-u", action="store_true", dest="uploader",
              help="show uploader of the source package")
    parser.add_option("-n", action="store_true", dest="name",
              help="show name of the source package")
    
    (options, args) = parser.parse_args()
    if options.changes is None:
        parser.print_help()
        parser.error("You must specify a .changes file.")
    
    chg = ChangesParser(options.changes)

    if options.files:
        print_files(chg)
    if options.version:
        print_version(chg)
    if options.uploader:
        print_uploader(chg)
    if options.name:
        print_name(chg)
