#!/usr/bin/python
# Copyright (c) 2005 Reinhard Tartler, Stefan Potyra
# Copyright (c) 2008 Siegfried-Angel Gevatter Pujals <rainct@ubuntu.com>
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
#
#     * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

import sys
import os
import re

from Config import Config
from register_user import register_user

class ChangesFile:

        def __init__(self, filename):
            self.filename = filename
            self.status = self.parse()
            
        def parse(self):
            try:
                f = open(self.filename)
                c = f.readlines()
                f.close()
            except IOError, e:
                print "IOError: %s\nBailing out." % e
                return False
            
            userregex = re.compile("Changed-By: [^<]+<(?P<email>[^>]+)>")
            m = userregex.search(''.join(c))
            if m is None:
                raise ValueError, 'Could not find maintainer email.'
            self.email = m.group('email')
            nameregex = re.compile("Source: (?P<name>[^\\s]+)")
            m = nameregex.search(''.join(c))
            if m is None:
                raise ValueError, 'Could not find source package name.'
            self.packagename = m.group('name')
            return True

def register_sourcepackage(c, name):
    """ register_sourcepackage(c, name) -> (sid, state)
    
    Return a tuple where the second element is 1 if the given source package is
    new or 0 if the package was already in REVU.
    
    """
    
    r = c.query("SELECT sid FROM SourcePackage WHERE NAME = '%s'" % name)
    if 0 == r.ntuples():
        c.query("INSERT INTO SourcePackage ( name ) VALUES ('%s')" % name)
        print "Source package %s has been registered in the database." % name
        r = c.query("SELECT currval ( 'SourcePackage_sid_seq' ) AS sid")
        state = 1
    else:
        print "Source package %s is already registered." % name
        state = 0
    return (r.dictresult()[0]['sid'], state)

def getUsid(c, email):
    sql = "SELECT usid FROM Users WHERE email = '%s'" % email
    r = c.query(sql)
    if 0 == r.ntuples():
        p = os.popen('pwgen 8 1')
        p = p.read().rstrip('\n')
        print "Uploader %s not found in Database" % email
        print "adding %s with 0 as gnupgid and password %s " % (email, p)
        register_user(c, email, '0', p)
        r = c.query(sql)
    return r.dictresult()[0]['usid']

def register_upload(c, sid, usid):
    r = c.query("INSERT INTO Upload ( sid, usid ) VALUES ( %d, %d )" % ( sid, usid ))
    r = c.query("SELECT currval ( 'Upload_upid_seq') as upid")
    return r.dictresult()[0]['upid']

def sendUploadMail(c, usid, upid, state):
    sql = """SELECT u.dateOfUpload, sp.name, urs.email FROM Upload u
         INNER JOIN SourcePackage sp ON u.sid=sp.sid
         INNER JOIN Users urs ON urs.usid=u.usid
         WHERE u.upid=%d""" % int(upid)
    q = c.query(sql)
    d = q.dictresult()
    
    packagetext = d[0]["name"]
    dateofupload = d[0]["dateofupload"]
    uploader = d[0]["email"]
    
    revulink = "http://revu.tauware.de/details.py?upid=%d" % upid
    
    if state == 1:
        newpkgtxt = "Package %s was newly uploaded to revu." % packagetext
        subject = "UPLOAD for %s" % packagetext
    else:
        newpkgtxt = "Package %s on revu was updated." % packagetext
        subject = "UPDATE for %s" % packagetext
    
    receiver = "motu-reviewers@lists.ubuntuwire.com"
    
    mailtext = newpkgtxt
    mailtext += "\nPackages was uploaded at: %s" % dateofupload
    mailtext += "\nPackage was uploaded from: %s" % uploader
    mailtext += "\nREVU-Link: %s" % revulink
    
    try:
        (i,o) = os.popen2('/usr/bin/mail -s "%s" %s -- -f "REVU Comment Watcher <motu-reviewers@lists.tauware.de>"' % (subject, receiver))
        i.write(mailtext)
        i.close()
        o.close()
    except:
        return False
    else:
        return True

def main(filename):
    cfg = Config()
    c = cfg.getConnection()
    cfile = ChangesFile(filename)
    if not cfile.status:
        return False
    print "Processing upload from %s uploading %s..." % (cfile.email, cfile.packagename)
    usid = getUsid(c, cfile.email)
    (sid, state) = register_sourcepackage(c, cfile.packagename)
    upid = register_upload(c, int(sid), int(usid))
    sendUploadMail(c, int(usid), upid, state)
    print "Done"

if __name__ == '__main__':
    if len(sys.argv) == 2:
        main(sys.argv[1])
    else:
         print "Usage: %s <changes file>" % sys.argv[0]
