# Copyright (c) 2008 Siegfried-Angel Gevatter Pujals <rainct@ubuntu.com>
#
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
# 
#     * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

from mod_python import psp

from Config import Config
from common import *

def _enumerate_list(list):
    """ enumerate_list(list) -> list
    
    Take a list of lists, insert an incremental number at the start of each of
    the many lists and return the changed list of lists.
    
    """
    
    for x in range(0, len(list)):
        list[x].insert(0, x+1)
    
    return list

def index(req):
    """ index(req) -> string
    
    This is stats.py's main function, called by Apache when the page is
    requested. It's function is to return the HTML for the statistics
    page, which contains rankings (eg., top commenters) and other information.
    
    """
    
    # Connect to the database
    config = Config(req)
    connection = config.getConnection()
    
    # Define some convenience aliases
    query = connection.query_result
    
    # Get all e-mails and their associated usid
    users = query('SELECT usid, email FROM users')
    
    # Get the info for the rankings
    top_commenters = query("SELECT users.email, COUNT(*) " +\
        "FROM comments, users WHERE comments.usid=users.usid " +\
        "GROUP BY users.email ORDER BY COUNT(*) DESC LIMIT 50")
    top_advocaters = query("SELECT users.email, COUNT(*) " +\
        "FROM comments, users " +\
        "WHERE comments.usid=users.usid AND comments.isadvocating='t' " +\
        "GROUP BY users.email ORDER BY COUNT(*) DESC LIMIT 50")
    
    vars = {
        'commenters': _enumerate_list(to_list(top_commenters)),
        'advocaters': _enumerate_list(to_list(top_advocaters)),
    }
    
    # Generate the HTML
    return psp.PSP(req, config.getTemplatesPath('statistics.html'), vars=vars)
