# Copyright (c) 2005 Reinhard Tartler, Stefan Potyra
# Copyright (c) 2008 Siegfried-Angel Gevatter Pujals <rainct@ubuntu.com>
#
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
# 
#     * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

import os
import cgi
from common import *
from Config import Config

def getUSID(c, login):
	sql = "SELECT usid FROM Users WHERE email = '%s'" % sanitize_username(login)
	r = c.query(sql)
	if r.ntuples() == 1:
		return r.dictresult()[0]["usid"]
	else:
		return None

def sendmailcomment(c, usid, upid, text, advocate):
	if advocate == 'yes':
		advocatetext="advocating"
		numofadvocates = 1
	else:
		advocatetext="not advocating"
		numofadvocates = 0

	sql = "SELECT email FROM Users WHERE usid=%d" % int(usid)
	q = c.query(sql)
	d = q.dictresult()
	usertext = d[0]["email"]

	sql = """SELECT u.dateOfUpload, sp.name, urs.email FROM Upload u 
		 INNER JOIN SourcePackage sp ON u.sid=sp.sid
		 INNER JOIN Users urs ON urs.usid=u.usid
		 WHERE u.upid=%d""" % int(upid)
	q = c.query(sql)
	d = q.dictresult()
	
	packagetext = d[0]["name"]
	dateofupload = d[0]["dateofupload"]
	uploader = d[0]["email"]


	sql = """SELECT count(*) as numad FROM Upload
		 INNER JOIN Comments ON Upload.upid=Comments.upid
  		 WHERE Comments.isAdvocating='t' AND Upload.upid=%d""" % upid
	q = c.query(sql)
	d = q.dictresult()

	numofadvocates = numofadvocates + d[0]["numad"]
	
	revulink = "http://revu.tauware.de/details.py?upid=%d" % upid

	subject = "COMMENT for %s" % packagetext
	receiver = "motu-reviewers@tauware.de"
	
	mailtext = "Comment for package %s (%s)" % (packagetext, advocatetext)
	mailtext += "\nNumber of Advocates: %d" % numofadvocates
	mailtext += "\nPackages was uploaded at: %s" % dateofupload
	mailtext += "\nPackage was uploaded from: %s" % uploader
	mailtext += "\nREVU-Link: %s" % revulink
	mailtext += "\n\n%s wrote:\n\n%s" % (usertext, text)

	try:
		(i,o) = os.popen2('/usr/bin/mail -s "%s" %s -- -f "REVU comment watcher <motu-reviewers@lists.tauware.de>"' % (subject, receiver))
		i.write(mailtext)
		i.close()
		o.close()
	except:
		pass

def addcomment(c, usid, upid, text, advocate):
	sendmailcomment(c, usid, upid, text, advocate)
	text = cgi.escape(text, True)
	text = text.replace("'", "&rsquo;")
	# escape single backslashes
	text = text.replace("\\", "\\\\")
	text = text.replace("\n", "<br>")
        r = c.query("INSERT INTO Comments (usid, upid, text, isAdvocating) VALUES ( %d, %d, '%s', '%s' ) " % (usid, upid, text, advocate))
	return r

def getUsidOfComment(c, cid):
	r = c.query("SELECT usid from Comments WHERE cid = %d" % int(cid))
	if r.ntuples() == 1:
		return r.dictresult()[0]["usid"]
	else:
		return None

def deletecomment(c, cid, currentlevel, currentusid):
	""" deletes Comment with checking if the User is allowed to do """
	commenterusid = getUsidOfComment(c, cid)
	if (currentlevel == 'Admin') or (currentusid == commenterusid):
		sql = 'DELETE FROM Comments WHERE cid = %d' % cid
		r = c.query(sql)

def _get_user_level(c, usid):
	sql = "SELECT accesslevel from users WHERE usid='%d'" % usid
	d = c.query(sql).dictresult()
	if len(d) < 1:
		return 'Unknown'
	else:
		return d[0]['accesslevel']

def addadvocate(c, cid, currentusid):
	commenterusid = getUsidOfComment(c, cid)
	if commenterusid == currentusid:
		sql = "UPDATE Comments SET isAdvocating = 't' WHERE cid = '%d'" % cid
		r = c.query(sql)
	else:
		curuser_level = _get_user_level(c, currentusid)
		comuser_level = _get_user_level(c, commenterusid)
		if curuser_level == 'Admin' and comuser_level in ('Reviewer', 'Admin'):
			sql = "UPDATE Comments SET isAdvocating = 't' WHERE cid='%d'" % cid
			r = c.query(sql)

def removeadvocate(c, cid, currentusid):
	commenterusid = getUsidOfComment(c, cid)
	if commenterusid == currentusid:
		sql = "UPDATE Comments SET isAdvocating = 'f' WHERE cid = '%d'" % cid
		r = c.query(sql)
	else:
		sql = "SELECT accesslevel from users WHERE usid='%d'" % currentusid
		d = c.query(sql).dictresult()
		if len(d) > 0 and d[0]["accesslevel"] == "Admin":
			sql = "UPDATE Comments SET isAdvocating = 'f' WHERE cid='%d'" % cid
			r = c.query(sql)

def actionlinks(c, cid, commentusid, currentuser, currentlevel, isadvocating, upid):
	
	links = []
	
	if currentlevel == 'Admin' or commentusid == currentuser:
		links = [ '<a href=details.py?upid=%d&amp;deletecomment=%d>[Delete]</a>' % (int(upid), int(cid)) ]
	
	if (currentlevel == 'Admin') or ((commentusid == currentuser) and currentlevel == 'Reviewer'):
		if isadvocating == 't':
		        links.append('<a href=details.py?upid=%d&amp;removeadvocate=%d>[Remove advocation]</a>' % (upid, cid))
		else:
			if _get_user_level(c, commentusid) in ('Reviewer', 'Admin'):
				links.append('<a href=details.py?upid=%d&amp;addadvocate=%d>[Add advocation]</a>' % (upid, cid))
	
	return ' '.join(links)

def _statusIcon(email, accesslevel, advocating):
	img = ''
	advocating = True if advocating == 't' else False
	if accesslevel in ('Reviewer', 'Admin'):
		if advocating:
			img = '<img alt="Advocated" title="Advocated by a MOTU" src="images/tick.png">'
		else:
			img = '<img alt="MOTU" title="MOTU (not advocating)" src="images/motu.png">'
	else:
		img = '<img alt="" title="Reviewer (non-MOTU)" src="images/bulb.png" width="16" height="16">'
	return img

def getcomments(c, currentuser, currentlevel, upid):
	if currentuser is None:
		currentuser = 0
		currentlevel = 0
	sql = """SELECT to_char(dateOfComment, 'Month DD  HH24:MI') as dateofc, cid, text, isAdvocating, usid, 
			email, accesslevel 
		 FROM Comments c NATURAL JOIN Users
		 WHERE upid = %d
		 ORDER By dateOfComment
	      """ % upid
	r = c.query(sql)
	back = [] 
	for row in r.dictresult():
		back.append( """
<table class="comments">
	<tr><th class="advocating">%s</th><th class="date">%s</th><th class="name">%s</th><th class="actions">%s</th></tr>
	<tr><td colspan="4"><span style="font-family: monospace">%s</span></td></tr>
</table>
""" % (
		_statusIcon(row['email'], row['accesslevel'], row['isadvocating']),
		row['dateofc'],
		display_email(row['email']),
		actionlinks(
			c, int(row['cid']), int(row['usid']), currentuser,
			currentlevel, row['isadvocating'], int(upid)
		),
		link_bugs(row['text']),
	))
	
	if back.count:
		return "\n".join(back)
	else:
		return ""
